/* Unit test suite for functions SystemParametersInfo and GetSystemMetrics.

   Copyright 2002 Andriy Palamarchuk

*/

#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include "wt_helper.h"

#define SPI_SETBEEP_REGKEY           _T("Control Panel\\Sound")
#define SPI_SETBEEP_VALNAME          _T("Beep")

HWND ghTestWnd;

DWORD WINAPI SysParamsThreadFunc( LPVOID lpParam );
LRESULT CALLBACK SysParamsTestWndProc( HWND hWnd, UINT msg, WPARAM wParam,
                                       LPARAM lParam );
int change_counter;
int change_last_param;
_TCHAR change_reg_section[MAX_PATH];

LRESULT CALLBACK SysParamsTestWndProc( HWND hWnd, UINT msg, WPARAM wParam,
                                       LPARAM lParam )
{
    switch(msg) {

    case WM_SETTINGCHANGE:
	change_counter++;
	change_last_param = wParam;
        _tcsncpy( change_reg_section, (LPTSTR) lParam,
                  sizeof( change_reg_section ) / sizeof(_TCHAR) );
        break;

    case WM_DESTROY:
        PostQuitMessage( 0 );
        break;

    default:
        return( DefWindowProc( hWnd, msg, wParam, lParam ) );
    }

    return 0;
}

/*
Performs testing for system parameters messages
params:
 - system parameter id
 - supposed value of the registry key
*/
void test_change_message( UINT action, _TCHAR *reg_section )  {
    ok( 1 == change_counter, _T("Each message is processed") );
    change_counter = 0;
    ok( action == change_last_param, _T("Correct action is processed") );
    change_last_param = 0;
    ok( !_tcscmp( reg_section, change_reg_section ),
        _T("Unexpected registry section") );
    _tcscpy( change_reg_section, _T(""));
}

/*
 * Tests the HKEY_CURRENT_USER subkey value.
 * The value should contain string value.
 *
 * Params:
 * lpsSubKey - subkey name
 * lpsRegName - registry entry name
 * lpsTestValue - value to test
 */
void _test_reg_key( _TCHAR *subKey, _TCHAR *valName, _TCHAR *testValue,
                    _TCHAR *file, int line ) {
    _TCHAR value[MAX_PATH] = _T("");
    DWORD valueLen = sizeof(value);
    DWORD type;
    HKEY hKey;

    RegOpenKey( HKEY_CURRENT_USER, subKey, &hKey );
    RegQueryValueEx( hKey, valName, NULL, &type, (LPBYTE)value, &valueLen );
    RegCloseKey( hKey );
    if(_tcscmp( testValue, value ))
        _tprintf( _T("subKey: %s, valName: \"%s\", testValue: \"%s\", value: \"%s\" "
                     "called from %s, line %i\n"),
                  subKey, valName, testValue, value, file, line);
    ok( !_tcscmp( testValue, value ),
        _T("System parameter registry key expected value") );
}

#define test_reg_key( subKey, valName, testValue ) \
    _test_reg_key( subKey, valName, testValue, _T(__FILE__), __LINE__ )

void test_SPI_SETBEEP( void )                   /*      2 */
{
    BOOL old_b;
    BOOL b;
    BOOL curr_val;

    SystemParametersInfo( SPI_GETBEEP, 0, &old_b, 0 );

    curr_val = TRUE;
    SystemParametersInfo( SPI_SETBEEP, curr_val, 0, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE );
    test_change_message( SPI_SETBEEP, _T("") );
    test_reg_key( SPI_SETBEEP_REGKEY,
                  SPI_SETBEEP_VALNAME,
                  curr_val ? _T("Yes") : _T("No") );
    SystemParametersInfo( SPI_GETBEEP, 0, &b, 0 );
    ok( curr_val == b, _T("retrieved value is the same as set") );
    
    /* is a message sent for the second change? */
    SystemParametersInfo( SPI_SETBEEP, curr_val, 0, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE );
    test_change_message( SPI_SETBEEP, _T("") );

    curr_val = FALSE;
    SystemParametersInfo( SPI_SETBEEP, curr_val, 0, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE );
    test_change_message( SPI_SETBEEP, _T("") );
    test_reg_key( SPI_SETBEEP_REGKEY,
                  SPI_SETBEEP_VALNAME,
                  curr_val ? _T("Yes") : _T("No") );
    SystemParametersInfo( SPI_GETBEEP, 0, &b, 0 );
    ok( curr_val == b, _T("retrieved value is the same as set") );
    ok( MessageBeep( MB_OK ), _T("Return value of MessageBeep when sound is disabled") );

    SystemParametersInfo( SPI_SETBEEP, old_b, 0, SPIF_UPDATEINIFILE );
}

/*
 * Registry entries for the system parameters.
 * Names are created by 'SET' flags names.
 * We assume that corresponding 'GET' entries use the same registry keys.
 */
DWORD WINAPI SysParamsThreadFunc( LPVOID lpParam )
{
    test_SPI_SETBEEP();                         /*      1 */

    SendMessage( ghTestWnd, WM_DESTROY, 0, 0 );
    return 0;
}

int main( void )
{
    WNDCLASS wc;
    MSG msg;
    HANDLE hThread;
    DWORD dwThreadId;
    HANDLE hInstance = GetModuleHandle( NULL );

    change_counter = 0;
    change_last_param = 0;
    _tcscpy( change_reg_section, _T("") );

    wc.lpszClassName = _T("SysParamsTestClass");
    wc.lpfnWndProc = SysParamsTestWndProc;
    wc.style = CS_OWNDC | CS_VREDRAW | CS_HREDRAW;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1 );
    wc.lpszMenuName = 0;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    RegisterClass( &wc );

    ghTestWnd = CreateWindow( _T("SysParamsTestClass"), _T("Test System Parameters Application"),
                             WS_OVERLAPPEDWINDOW, 0, 0, 100, 100, NULL, NULL, hInstance, NULL );
    ShowWindow( ghTestWnd, SW_SHOWNORMAL );

    hThread = CreateThread( NULL, 0, SysParamsThreadFunc, 0, 0, &dwThreadId );
   
    if (hThread == NULL)
    {
        _tprintf( _T("CreateThread failed.") );
        exit( 1 );
    }
    CloseHandle( hThread );

    while( GetMessage( &msg, NULL, 0, 0 )) {
        TranslateMessage( &msg );
        DispatchMessage( &msg );
    }

    end_tests();
    return msg.wParam;
}
